@@ -3,10 +3,12 @@ import functools
 import pathlib
 import typing as t
 import warnings
+from typing import Optional
 
 import a2.dataset.radar
 import a2.dataset.tweets
 import a2.plotting.axes_utils
+import a2.plotting.figures
 import a2.plotting.utils_plotting
 import a2.utils.checks
 import a2.utils.constants
@@ -22,7 +24,7 @@ TypeKeyRadar = object
 
 def get_info_string_tweets(
     ds: xarray.Dataset,
-    fields: t.List[str] = None,
+    fields: Optional[t.List[str]] = None,
     add: t.Optional[t.List[str]] = None,
 ) -> str:
     """
@@ -71,16 +73,16 @@ def get_info_string_tweets(
 def plot_precipiation_map(
     ds_precipitation: xarray.Dataset,
     ds_tweets: xarray.Dataset,
-    key_time: str = "created_at_h",
+    key_time: str = "time_half",
     key_longitude: str = "longitude_rounded",
     key_latitude: str = "latitude_rounded",
-    key_tp: str = "tp",
+    key_tp: str = "tp_h",
     n_time: int = 1,
     delta_time: int = 1,
     delta_time_units: str = "h",
     delta_longitude: float = 1,
     delta_latitude: float = 1,
-    filename: t.Union[str, pathlib.Path] = None,
+    filename: Optional[t.Union[str, pathlib.Path]] = None,
     print_additional: t.Optional[t.List[str]] = None,
     add_time_before_plot: t.Optional[pd.Timedelta] = None,
     return_plots: bool = False,
@@ -157,7 +159,7 @@ def plot_precipiation_map(
         )
         axes[index, n_time].set_title(title)
     fig.tight_layout()
-    a2.plotting.utils_plotting.save_figure(fig, filename)
+    a2.plotting.figures.save_figure(fig, filename)
     if return_plots:
         return axes, plots
     return axes
@@ -167,10 +169,10 @@ def circle_scatter(
     ax: plt.axes,
     x: np.ndarray,
     y: np.ndarray,
-    color: t.Union[str, list, np.ndarray] = None,
+    color: Optional[t.Union[str, list, np.ndarray]] = None,
     radius: t.Union[float, list, np.ndarray] = 1,
     fill: bool = True,
-    cmap: matplotlib.colormaps = None,
+    cmap: matplotlib.cm.ColormapRegistry | None = None,
     norm=None,
     alpha: float = 1,
     show_colorbar: bool = False,
@@ -218,8 +220,8 @@ def _plot_radar_map_with_tweets(
     index: np.datetime64,
     ds: xarray.Dataset,
     path_to_dapceda: t.Optional[pathlib.Path] = None,
-    key_tweets: TypeKeyTweets = None,
-    key_radar: TypeKeyRadar = None,
+    key_tweets: Optional[TypeKeyTweets] = None,
+    key_radar: Optional[TypeKeyRadar] = None,
     cumulative_radar: bool = False,
     cumulative_delta_time: float = 1,
     cumulative_delta_time_units: str = "h",
@@ -334,13 +336,13 @@ def _plot_tp_station_tweets(
     index,
     ds: xarray.Dataset,
     df_stations: pd.DataFrame,
-    key_tweets: TypeKeyTweets = None,
+    key_tweets: Optional[TypeKeyTweets] = None,
     key_station_tp: str = "prcp_amt",
     key_station_time: str = "ob_end_time",
-    vmin: float = None,
-    vmax: float = None,
-    xlim: t.Sequence = None,
-    ylim: t.Sequence = None,
+    vmin: Optional[float] = None,
+    vmax: Optional[float] = None,
+    xlim: Optional[t.Sequence] = None,
+    ylim: Optional[t.Sequence] = None,
     time_offset_station: float = 0.5,
     time_offset_station_units: str = "h",
     fontsize: float = 20,
@@ -350,8 +352,8 @@ def _plot_tp_station_tweets(
     circle_size_constant: bool = False,
     circle_size: float = 0.1,
     radius_station: float = 0.1,
-    ax: plt.axes = None,
-    fig: plt.figure = None,
+    ax: Optional[plt.axes] = None,
+    fig: Optional[plt.figure] = None,
 ):
     """
     Plot station precipitation values and values assigned to Tweets
@@ -465,11 +467,11 @@ class MapsPlotter:
         selector: t.Callable,
         figsize_default: float = 15,
         figsize: t.Optional[t.List[float]] = None,
-        key_time: str = None,
+        key_time: Optional[str] = None,
         choice_type: str = "random",
-        increment_time_value: np.datetime64 = None,
-        increment_time_delta: float = None,
-        increment_time_delta_units: str = None,
+        increment_time_value: Optional[np.datetime64] = None,
+        increment_time_delta: Optional[float] = None,
+        increment_time_delta_units: Optional[str] = None,
         aspect: str = "auto",
         height_ratios: t.Optional[t.List] = None,
     ) -> None:
@@ -578,8 +580,8 @@ def plot_radar_map_with_tweets(
     figsize: t.Optional[t.List] = None,
     figsize_default: float = 30,
     path_to_dapceda: t.Optional[pathlib.Path] = None,
-    key_tweets: TypeKeyTweets = None,
-    key_radar: TypeKeyRadar = None,
+    key_tweets: Optional[TypeKeyTweets] = None,
+    key_radar: Optional[TypeKeyRadar] = None,
     vmin: t.Optional[float] = None,
     vmax: t.Optional[float] = None,
     xlim: t.Optional[list] = None,
@@ -589,8 +591,8 @@ def plot_radar_map_with_tweets(
     selection_delta_time: float = 10,
     selection_delta_time_units: str = "m",
     selection_key_twitter_time: str = "time_radar",
-    selector_use_limits: t.Sequence = None,
-    increment_time_value: np.datetime64 = None,
+    selector_use_limits: Optional[t.Sequence] = None,
+    increment_time_value: Optional[np.datetime64] = None,
     increment_time_delta: float = 1,
     increment_time_delta_units: str = "h",
     colormap: str = "viridis",
@@ -693,19 +695,19 @@ def plot_tp_station_tweets(
     grid_shape: t.Optional[t.Tuple] = None,
     figsize: t.Optional[t.List] = None,
     figsize_default: float = 30,
-    key_tweets: TypeKeyTweets = None,
+    key_tweets: Optional[TypeKeyTweets] = None,
     key_station_tp: str = "prcp_amt",
     key_station_time: str = "ob_end_time",
-    vmin: float = None,
-    vmax: float = None,
-    xlim: t.Sequence = None,
-    ylim: t.Sequence = None,
+    vmin: Optional[float] = None,
+    vmax: Optional[float] = None,
+    xlim: Optional[t.Sequence] = None,
+    ylim: Optional[t.Sequence] = None,
     time_offset_station: float = 0.5,
     time_offset_station_units: str = "h",
     selector_key_time: str = "created_at",
     selection_delta_time: float = 0.5,
     selection_delta_time_units: str = "h",
-    selector_use_limits: t.Sequence = None,
+    selector_use_limits: Optional[t.Sequence] = None,
     choice_type: str = "random_time",
     choice_time: str = "time_half",
     increment_time_value=None,
@@ -853,13 +855,13 @@ def time_selector(
 
 def plot_radar_from_time(
     time,
-    path_to_dapceda: pathlib.Path = None,
+    path_to_dapceda: Optional[pathlib.Path] = None,
     cumulative: bool = False,
     time_delta: float = 1,
     time_delta_units: str = "h",
-    vmin: float = None,
-    vmax: float = None,
-    ax: plt.axes = None,
+    vmin: Optional[float] = None,
+    vmax: Optional[float] = None,
+    ax: Optional[plt.axes] = None,
     xlim: t.Optional[list] = None,
     ylim: t.Optional[list] = None,
 ) -> xarray.Dataset:
