@@ -1,5 +1,5 @@
 import a2.training.evaluate_hugging
+import a2.training.model_configs
 import a2.training.tracking
 import a2.training.tracking_hugging
-import a2.training.training_deep500
 import a2.training.training_hugging
