@@ -4,8 +4,10 @@ import pprint
 import resource
 import time
 
+import a2.utils.utils
 import numpy as np
-import torch
+
+torch = a2.utils.utils._import_torch(__file__)
 
 
 @dataclasses.dataclass(frozen=True)
@@ -29,6 +31,8 @@ def current_time():
 
 
 class Timer:
+    """Interface has analagous implementation to deep500 timer. Therefore not all variables used"""
+
     def __init__(self, print_all_single_time_stats=True, debug=True):
         self.times_archive = {}
         self.times_running = {}
@@ -57,8 +61,6 @@ class Timer:
             logging.warning(f"Attempting to finish timer type {time_type}, which was never started!")
 
     def complete_all(self):
-        # for _type in self.times_running.copy().keys():
-        #     self.end(_type)
         pass
 
     def print_all_time_stats(self):
@@ -77,27 +79,32 @@ class Timer:
 
 
 def get_max_memory_usage():
-    """In bytes"""
+    """Max RAM memory used in bytes"""
     return resource.getrusage(resource.RUSAGE_SELF).ru_maxrss * 1000
 
 
-def init_cuda():
-    if torch.cuda.is_available():
-        torch.cuda.init()
+class CudaMemoryMonitor:
+    def __init__(self) -> None:
+        self.init_cuda()
 
+    def init_cuda(self):
+        if torch.cuda.is_available():
+            torch.cuda.init()
+        else:
+            logging.info("Cuda initialization failed, as cuda is not available.")
 
-def reset_cuda_memory_monitoring():
-    for i_cuda in range(torch.cuda.device_count()):
-        torch.cuda.reset_peak_memory_stats(i_cuda)
+    def reset_cuda_memory_monitoring(self):
+        for i_cuda in range(torch.cuda.device_count()):
+            torch.cuda.reset_peak_memory_stats(i_cuda)
 
-
-def get_cuda_memory_usage(log_message):
-    logging.info("CUDA memory logging....\n")
-    for i_cuda in range(torch.cuda.device_count()):
-        logging.info(f"{log_message}: Cuda device {i_cuda} report:\n")
-        pprint.pprint(torch.cuda.memory_stats(i_cuda))
-        logging.info(f"Report done! for Cuda device {i_cuda}\n")
-        logging.info(f"Gpu memory currently allocated: {torch.cuda.memory_allocated(i_cuda)/1e9} GB.")
-        logging.info(f"Gpu max memory allocated: {torch.cuda.max_memory_allocated(i_cuda)/1e9} GB.")
-        logging.info(f"Gpu memory currently reserved: {torch.cuda.memory_reserved(i_cuda)/1e9} GB.")
-        logging.info(f"Gpu max memory reserved: {torch.cuda.max_memory_reserved(i_cuda)/1e9} GB.")
+    def get_cuda_memory_usage(self, log_message, style="verbose"):
+        logging.info("CUDA memory logging....\n")
+        for i_cuda in range(torch.cuda.device_count()):
+            logging.info(f"{log_message}: Cuda device {i_cuda} report:\n")
+            if style == "verbose":
+                pprint.pprint(torch.cuda.memory_stats(i_cuda))
+                logging.info(f"Report done! for Cuda device {i_cuda}\n")
+            logging.info(f"Gpu memory currently allocated: {torch.cuda.memory_allocated(i_cuda)/1e9} GB.")
+            logging.info(f"Gpu max memory allocated: {torch.cuda.max_memory_allocated(i_cuda)/1e9} GB.")
+            logging.info(f"Gpu memory currently reserved: {torch.cuda.memory_reserved(i_cuda)/1e9} GB.")
+            logging.info(f"Gpu max memory reserved: {torch.cuda.max_memory_reserved(i_cuda)/1e9} GB.")
